@@ -1,8 +1,4 @@
 //! Library that provides definitions for daemon related features in streamduck
-
-pub use streamduck_core::versions;
-pub use streamduck_core::modules;
-
 pub mod socket;
 pub mod core_manager;
 pub mod config;
\ No newline at end of file
