@@ -6,11 +6,12 @@ use streamduck_core::versions::SOCKET_API;
 use crate::core_manager::CoreManager;
 use crate::socket::{check_packet_for_data, parse_packet_to_data, send_packet, SocketData, SocketHandle, SocketListener, SocketPacket};
 use strum_macros::Display;
-use streamduck_core::core::methods::{CoreHandle, get_current_screen, get_stack, set_brightness};
+use streamduck_core::core::button::Button;
+use streamduck_core::core::methods::{clear_button, CoreHandle, get_button, get_current_screen, get_stack, load_panels, pop_screen, push_screen, replace_screen, save_panels, set_brightness, set_button};
 use streamduck_core::core::RawButtonPanel;
 use streamduck_core::modules::{ModuleManager, PluginMetadata};
 use streamduck_core::modules::components::ComponentDefinition;
-use streamduck_core::util::panel_to_raw;
+use streamduck_core::util::{button_to_raw, make_button_unique, make_panel_unique, panel_to_raw};
 use crate::config::{Config, ConfigError};
 
 /// Listener for daemon types
@@ -46,6 +47,14 @@ impl SocketListener for DaemonListener {
         // Panel management
         process_for_type::<GetStack>(self, socket, &packet);
         process_for_type::<GetCurrentScreen>(self, socket, &packet);
+        process_for_type::<GetButton>(self, socket, &packet);
+        process_for_type::<SetButton>(self, socket, &packet);
+        process_for_type::<ClearButton>(self, socket, &packet);
+        process_for_type::<PushScreen>(self, socket, &packet);
+        process_for_type::<PopScreen>(self, socket, &packet);
+        process_for_type::<ReplaceScreen>(self, socket, &packet);
+        process_for_type::<ResetStack>(self, socket, &packet);
+        process_for_type::<CommitChangesToConfig>(self, socket, &packet);
     }
 }
 
@@ -61,7 +70,7 @@ fn process_for_type<T: DaemonRequest + SocketData>(listener: &DaemonListener, ha
 
 // Version
 
-/// Request data for socket API version
+/// Request for socket API version
 #[derive(Serialize, Deserialize)]
 pub struct SocketAPIVersion {
     pub version: String
@@ -83,7 +92,7 @@ impl DaemonRequest for SocketAPIVersion {
 
 // Device management
 
-/// Request data for getting device list
+/// Request for getting device list
 #[derive(Serialize, Deserialize)]
 pub struct ListDevices {
     pub devices: Vec<Device>
@@ -161,7 +170,7 @@ impl DeviceType {
     }
 }
 
-/// Request data for getting a device
+/// Request for getting a device
 #[derive(Serialize, Deserialize)]
 pub struct GetDevice {
     pub serial_number: String
@@ -171,6 +180,7 @@ impl SocketData for GetDevice {
     const NAME: &'static str = "get_device";
 }
 
+/// Response of GetDevice request
 #[derive(Serialize, Deserialize)]
 pub enum GetDeviceResult {
     /// Sent when device is found
@@ -204,7 +214,7 @@ impl DaemonRequest for GetDevice {
 }
 
 
-/// Request data for adding a device
+/// Request for adding a device
 #[derive(Serialize, Deserialize)]
 pub struct AddDevice {
     pub serial_number: String,
@@ -214,6 +224,7 @@ impl SocketData for AddDevice {
     const NAME: &'static str = "add_device";
 }
 
+/// Response of AddDevice request
 #[derive(Serialize, Deserialize)]
 pub enum AddDeviceResult {
     /// Sent if device is already added
@@ -250,7 +261,7 @@ impl DaemonRequest for AddDevice {
     }
 }
 
-/// Request data for removing a device
+/// Request for removing a device
 #[derive(Serialize, Deserialize)]
 pub struct RemoveDevice {
     pub serial_number: String,
@@ -260,6 +271,7 @@ impl SocketData for RemoveDevice {
     const NAME: &'static str = "remove_device";
 }
 
+/// Response of RemoveDevice request
 #[derive(Serialize, Deserialize)]
 pub enum RemoveDeviceResult {
     /// Sent if device already wasn't added
@@ -287,6 +299,7 @@ impl DaemonRequest for RemoveDevice {
 }
 
 // Device configuration
+/// Request for reloading all device configs
 #[derive(Serialize, Deserialize)]
 pub enum ReloadDeviceConfigsResult {
     /// Sent if error happened while reloading configs
@@ -316,11 +329,13 @@ impl DaemonRequest for ReloadDeviceConfigsResult {
     }
 }
 
+/// Request for reloading device config for specific device
 #[derive(Serialize, Deserialize)]
 pub struct ReloadDeviceConfig {
     pub serial_number: String
 }
 
+/// Response of ReloadDeviceConfig request
 #[derive(Serialize, Deserialize)]
 pub enum ReloadDeviceConfigResult {
     /// Sent if error happened while reloading configs
@@ -361,6 +376,7 @@ impl DaemonRequest for ReloadDeviceConfig {
     }
 }
 
+/// Request for saving all device configs
 #[derive(Serialize, Deserialize)]
 pub enum SaveDeviceConfigsResult {
     /// Sent if error happened while saving configs
@@ -390,11 +406,13 @@ impl DaemonRequest for SaveDeviceConfigsResult {
     }
 }
 
+/// Request for saving device config for specific device
 #[derive(Serialize, Deserialize)]
 pub struct SaveDeviceConfig {
     pub serial_number: String,
 }
 
+/// Response of SaveDeviceConfig request
 #[derive(Serialize, Deserialize)]
 pub enum SaveDeviceConfigResult {
     /// Sent if error happened while saving config
@@ -435,12 +453,14 @@ impl DaemonRequest for SaveDeviceConfig {
     }
 }
 
+/// Request for setting device's brightness
 #[derive(Serialize, Deserialize)]
 pub struct SetBrightness {
     pub serial_number: String,
     pub brightness: u8,
 }
 
+/// Response of SetBrightness request
 #[derive(Serialize, Deserialize)]
 pub enum SetBrightnessResult {
     /// Sent if device wasn't found
@@ -482,6 +502,7 @@ impl DaemonRequest for SetBrightness {
 }
 
 // Module management
+/// Request for getting all loaded modules
 #[derive(Serialize, Deserialize)]
 pub struct ListModules {
     pub modules: Vec<PluginMetadata>
@@ -506,6 +527,7 @@ impl DaemonRequest for ListModules {
     }
 }
 
+/// Request for getting all components defined by all modules
 #[derive(Serialize, Deserialize)]
 pub struct ListComponents {
     /// Hashmap of module name to component map
@@ -532,11 +554,13 @@ impl DaemonRequest for ListComponents {
 }
 
 // Panel management
+/// Request for getting current stack on a device
 #[derive(Serialize, Deserialize)]
 pub struct GetStack {
     pub serial_number: String
 }
 
+/// Response of GetStack request
 #[derive(Serialize, Deserialize)]
 pub enum GetStackResult {
     /// Sent if device wasn't found
@@ -575,12 +599,13 @@ impl DaemonRequest for GetStack {
     }
 }
 
-
+/// Request for getting current screen on a device
 #[derive(Serialize, Deserialize)]
 pub struct GetCurrentScreen {
     pub serial_number: String
 }
 
+/// Response of GetCurrentScreen request
 #[derive(Serialize, Deserialize)]
 pub enum GetCurrentScreenResult {
     /// Sent if there's no screen
@@ -589,7 +614,7 @@ pub enum GetCurrentScreenResult {
     /// Sent if device wasn't found
     DeviceNotFound,
 
-    /// Sent if successfully got stack
+    /// Sent if successfully got the stack
     Screen(RawButtonPanel)
 }
 
@@ -617,4 +642,348 @@ impl DaemonRequest for GetCurrentScreen {
             }
         }
     }
+}
+
+/// Request for getting a button from current screen on a device
+#[derive(Serialize, Deserialize)]
+pub struct GetButton {
+    pub serial_number: String,
+    pub key: u8
+}
+
+/// Response of GetButton request
+#[derive(Serialize, Deserialize)]
+pub enum GetButtonResult {
+    /// Sent if device wasn't found
+    DeviceNotFound,
+
+    /// Sent if there's no button there
+    NoButton,
+
+    /// Sent if successfully got the button
+    Button(Button)
+}
+
+impl SocketData for GetButton {
+    const NAME: &'static str = "get_button";
+}
+
+impl SocketData for GetButtonResult {
+    const NAME: &'static str = "get_button";
+}
+
+impl DaemonRequest for GetButton {
+    fn process(listener: &DaemonListener, handle: SocketHandle, packet: &SocketPacket) {
+        if let Ok(request) = parse_packet_to_data::<GetButton>(packet) {
+            if let Some(device) = listener.core_manager.get_device(&request.serial_number) {
+                let wrapped_core = CoreHandle::wrap(device.core);
+
+                if let Some(button) = get_button(&wrapped_core, request.key) {
+                    send_packet(handle, packet, &GetButtonResult::Button(button_to_raw(&button))).ok();
+                } else {
+                    send_packet(handle, packet, &GetButtonResult::NoButton).ok();
+                }
+            } else {
+                send_packet(handle, packet, &GetButtonResult::DeviceNotFound).ok();
+            }
+        }
+    }
+}
+
+/// Request for setting a button on current screen on a device
+#[derive(Serialize, Deserialize)]
+pub struct SetButton {
+    pub serial_number: String,
+    pub key: u8,
+    pub button: Button
+}
+
+/// Response of SetButton request
+#[derive(Serialize, Deserialize)]
+pub enum SetButtonResult {
+    /// Sent if there's no screen to set to
+    NoScreen,
+
+    /// Sent if device wasn't found
+    DeviceNotFound,
+
+    /// Sent if successfully set the button
+    Set
+}
+
+impl SocketData for SetButton {
+    const NAME: &'static str = "set_button";
+}
+
+impl SocketData for SetButtonResult {
+    const NAME: &'static str = "set_button";
+}
+
+impl DaemonRequest for SetButton {
+    fn process(listener: &DaemonListener, handle: SocketHandle, packet: &SocketPacket) {
+        if let Ok(request) = parse_packet_to_data::<SetButton>(packet) {
+            if let Some(device) = listener.core_manager.get_device(&request.serial_number) {
+                let wrapped_core = CoreHandle::wrap(device.core);
+
+                if set_button(&wrapped_core, request.key, make_button_unique(request.button)) {
+                    send_packet(handle, packet, &SetButtonResult::Set).ok();
+                } else {
+                    send_packet(handle, packet, &SetButtonResult::NoScreen).ok();
+                }
+            } else {
+                send_packet(handle, packet, &SetButtonResult::DeviceNotFound).ok();
+            }
+        }
+    }
+}
+
+/// Request for clearing a button on current screen on a device
+#[derive(Serialize, Deserialize)]
+pub struct ClearButton {
+    pub serial_number: String,
+    pub key: u8,
+}
+
+/// Response of ClearButton request
+#[derive(Serialize, Deserialize)]
+pub enum ClearButtonResult {
+    /// Sent if device wasn't found
+    DeviceNotFound,
+
+    /// Sent if there's no screen, or there's no button to clear
+    FailedToClear,
+
+    /// Sent if successfully set the button
+    Cleared
+}
+
+impl SocketData for ClearButton {
+    const NAME: &'static str = "clear_button";
+}
+
+impl SocketData for ClearButtonResult {
+    const NAME: &'static str = "clear_button";
+}
+
+impl DaemonRequest for ClearButton {
+    fn process(listener: &DaemonListener, handle: SocketHandle, packet: &SocketPacket) {
+        if let Ok(request) = parse_packet_to_data::<ClearButton>(packet) {
+            if let Some(device) = listener.core_manager.get_device(&request.serial_number) {
+                let wrapped_core = CoreHandle::wrap(device.core);
+
+                if clear_button(&wrapped_core, request.key) {
+                    send_packet(handle, packet, &ClearButtonResult::Cleared).ok();
+                } else {
+                    send_packet(handle, packet, &ClearButtonResult::FailedToClear).ok();
+                }
+            } else {
+                send_packet(handle, packet, &ClearButtonResult::DeviceNotFound).ok();
+            }
+        }
+    }
+}
+
+/// Request for pushing a new screen on a device
+#[derive(Serialize, Deserialize)]
+pub struct PushScreen {
+    pub serial_number: String,
+    pub screen: RawButtonPanel
+}
+
+/// Response of PushScreen request
+#[derive(Serialize, Deserialize)]
+pub enum PushScreenResult {
+    /// Sent if device wasn't found
+    DeviceNotFound,
+
+    /// Sent if successfully pushed a screen
+    Pushed
+}
+
+impl SocketData for PushScreen {
+    const NAME: &'static str = "push_screen";
+}
+
+impl SocketData for PushScreenResult {
+    const NAME: &'static str = "push_screen";
+}
+
+impl DaemonRequest for PushScreen {
+    fn process(listener: &DaemonListener, handle: SocketHandle, packet: &SocketPacket) {
+        if let Ok(request) = parse_packet_to_data::<PushScreen>(packet) {
+            if let Some(device) = listener.core_manager.get_device(&request.serial_number) {
+                let wrapped_core = CoreHandle::wrap(device.core);
+
+                push_screen(&wrapped_core, make_panel_unique(request.screen));
+                send_packet(handle, packet, &PushScreenResult::Pushed).ok();
+            } else {
+                send_packet(handle, packet, &PushScreenResult::DeviceNotFound).ok();
+            }
+        }
+    }
+}
+
+/// Request for popping top-most screen on a device
+#[derive(Serialize, Deserialize)]
+pub struct PopScreen {
+    pub serial_number: String
+}
+
+/// Response of PopScreen request
+#[derive(Serialize, Deserialize)]
+pub enum PopScreenResult {
+    /// Sent if device wasn't found
+    DeviceNotFound,
+
+    /// Sent if successfully popped a screen
+    Popped
+}
+
+impl SocketData for PopScreen {
+    const NAME: &'static str = "pop_screen";
+}
+
+impl SocketData for PopScreenResult {
+    const NAME: &'static str = "pop_screen";
+}
+
+impl DaemonRequest for PopScreen {
+    fn process(listener: &DaemonListener, handle: SocketHandle, packet: &SocketPacket) {
+        if let Ok(request) = parse_packet_to_data::<PopScreen>(packet) {
+            if let Some(device) = listener.core_manager.get_device(&request.serial_number) {
+                let wrapped_core = CoreHandle::wrap(device.core);
+
+                pop_screen(&wrapped_core);
+                send_packet(handle, packet, &PopScreenResult::Popped).ok();
+            } else {
+                send_packet(handle, packet, &PopScreenResult::DeviceNotFound).ok();
+            }
+        }
+    }
+}
+
+/// Request for replacing a screen on a device
+#[derive(Serialize, Deserialize)]
+pub struct ReplaceScreen {
+    pub serial_number: String,
+    pub screen: RawButtonPanel
+}
+
+/// Response of ReplaceScreen request
+#[derive(Serialize, Deserialize)]
+pub enum ReplaceScreenResult {
+    /// Sent if device wasn't found
+    DeviceNotFound,
+
+    /// Sent if successfully replaced the screen
+    Replaced
+}
+
+impl SocketData for ReplaceScreen {
+    const NAME: &'static str = "replace_screen";
+}
+
+impl SocketData for ReplaceScreenResult {
+    const NAME: &'static str = "replace_screen";
+}
+
+impl DaemonRequest for ReplaceScreen {
+    fn process(listener: &DaemonListener, handle: SocketHandle, packet: &SocketPacket) {
+        if let Ok(request) = parse_packet_to_data::<ReplaceScreen>(packet) {
+            if let Some(device) = listener.core_manager.get_device(&request.serial_number) {
+                let wrapped_core = CoreHandle::wrap(device.core);
+
+                replace_screen(&wrapped_core, make_panel_unique(request.screen));
+                send_packet(handle, packet, &ReplaceScreenResult::Replaced).ok();
+            } else {
+                send_packet(handle, packet, &ReplaceScreenResult::DeviceNotFound).ok();
+            }
+        }
+    }
+}
+
+/// Request for replacing a screen on a device
+#[derive(Serialize, Deserialize)]
+pub struct ResetStack {
+    pub serial_number: String,
+    pub screen: RawButtonPanel
+}
+
+/// Response of ReplaceScreen request
+#[derive(Serialize, Deserialize)]
+pub enum ResetStackResult {
+    /// Sent if device wasn't found
+    DeviceNotFound,
+
+    /// Sent if successfully reset stack with a screen
+    Reset
+}
+
+impl SocketData for ResetStack {
+    const NAME: &'static str = "reset_stack";
+}
+
+impl SocketData for ResetStackResult {
+    const NAME: &'static str = "reset_stack";
+}
+
+impl DaemonRequest for ResetStack {
+    fn process(listener: &DaemonListener, handle: SocketHandle, packet: &SocketPacket) {
+        if let Ok(request) = parse_packet_to_data::<ResetStack>(packet) {
+            if let Some(device) = listener.core_manager.get_device(&request.serial_number) {
+                let wrapped_core = CoreHandle::wrap(device.core);
+
+                load_panels(&wrapped_core, make_panel_unique(request.screen));
+                send_packet(handle, packet, &ResetStackResult::Reset).ok();
+            } else {
+                send_packet(handle, packet, &ResetStackResult::DeviceNotFound).ok();
+            }
+        }
+    }
+}
+
+/// Request for committing all changes of the stack to device config
+#[derive(Serialize, Deserialize)]
+pub struct CommitChangesToConfig {
+    pub serial_number: String
+}
+
+#[derive(Serialize, Deserialize)]
+pub enum CommitChangesToConfigResult {
+    /// Sent if device wasn't found
+    DeviceNotFound,
+
+    /// Sent if successfully committed changes
+    Committed
+}
+
+impl SocketData for CommitChangesToConfig {
+    const NAME: &'static str = "commit_changes";
+}
+
+impl SocketData for CommitChangesToConfigResult {
+    const NAME: &'static str = "commit_changes";
+}
+
+impl DaemonRequest for CommitChangesToConfig {
+    fn process(listener: &DaemonListener, handle: SocketHandle, packet: &SocketPacket) {
+        if let Ok(request) = parse_packet_to_data::<CommitChangesToConfig>(packet) {
+            if let Some(device) = listener.core_manager.get_device(&request.serial_number) {
+                let wrapped_core = CoreHandle::wrap(device.core);
+
+                let stack = save_panels(&wrapped_core);
+
+                if let Some(mut config) = listener.config.get_device_config(&request.serial_number) {
+                    config.layout = panel_to_raw(&stack);
+                    listener.config.set_device_config(&request.serial_number, config);
+
+                    send_packet(handle, packet, &CommitChangesToConfigResult::Committed).ok();
+                } else {
+                    send_packet(handle, packet, &CommitChangesToConfigResult::DeviceNotFound).ok();
+                }
+            } else {
+                send_packet(handle, packet, &CommitChangesToConfigResult::DeviceNotFound).ok();
+            }
+        }
+    }
 }
\ No newline at end of file
